package com.miya.mpos.broadcast;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.NetworkInfo;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.text.TextUtils;
import android.widget.Toast;

import com.alibaba.android.arouter.launcher.ARouter;
import com.blankj.utilcode.util.AppUtils;
import com.blankj.utilcode.util.LogUtils;
import com.blankj.utilcode.util.SPUtils;
import com.miya.iot.sco.constance.ARouterPathConstant;
import com.miya.iot.sco.core.datasource.DataSourceManager;
import com.miya.iot.sco.core.sharedpreferences.PreferencesEngine;
import com.miya.iot.sco.core.utils.ActivityStackControlUtil;
import com.miya.iot.sco.core.utils.ContextUtils;
import com.miya.iot.sco.core.utils.DeviceUtil;
import com.miya.iot.sco.core.utils.LogFileUtils;
import com.miya.iot.sco.impl.component.ScoComponentProviderManager;
import com.miya.iot.sco.printer.PrintManager;
import com.miya.iot.sco.widget.CenterToasty;
import com.miya.mpos.views.MposHomeActivity;
import com.miya.mpos.views.MposLoginActivity;
import com.miya.mpos.widget.dialog.MessagePromptDialog;

import java.lang.ref.SoftReference;

import static com.blankj.utilcode.util.ActivityUtils.startActivity;

/**
 * todo 这个功能如何做个开关？
 */
public class WIFIStateChangedBroadcastReceiver extends BroadcastReceiver {

    private MessagePromptDialog mMessagePromptDialog;
    //这里必须要用static 因为每次广播来都是一个新的实例
    private static long mLastTime;

    @Override
    public void onReceive(Context context, Intent intent) {

        if (null == intent || TextUtils.isEmpty(intent.getAction())) {
            return;
        }

        if (intent.getAction().equals(WifiManager.NETWORK_STATE_CHANGED_ACTION)) {
            NetworkInfo info = intent.getParcelableExtra(WifiManager.EXTRA_NETWORK_INFO);
            if (info.getState().equals(NetworkInfo.State.CONNECTED)) {
                WifiManager wifiManager = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
                WifiInfo wifiInfo = wifiManager.getConnectionInfo();
                LogFileUtils.writeLog(ContextUtils.getContext(), "连接到WIFI " + wifiInfo.getSSID());
                LogUtils.e("###连接到WIFI");
                if (isWifiChange(context)) go2Login(context);
            }
        }

        if (intent.getAction().equals(WifiManager.WIFI_STATE_CHANGED_ACTION)) {
            int wifistate = intent.getIntExtra(WifiManager.EXTRA_WIFI_STATE, WifiManager.WIFI_STATE_DISABLED);

            if (wifistate == WifiManager.WIFI_STATE_DISABLED) {
                //注意重复执行
                LogFileUtils.writeLog(ContextUtils.getContext(),"WIFI关闭");
                LogUtils.e("###WIFI关闭");
                go2Login(context);
            }

            if (wifistate == WifiManager.WIFI_STATE_ENABLED) {
                LogFileUtils.writeLog(ContextUtils.getContext(), "WIFI开启");
                LogUtils.e("###WIFI开启");
                if (isWifiChange(context)) go2Login(context);
            }
        }
    }


    private void go2Login(Context context) {
        if((System.currentTimeMillis() - mLastTime) < 3000)return;
        mLastTime = System.currentTimeMillis();

        if (!AppUtils.isAppForeground()) return;

        Activity currentActivity = ActivityStackControlUtil.currentActivity();
        if (currentActivity == null || currentActivity.isFinishing() || currentActivity instanceof MposLoginActivity) {
            return;
        }

        //这里的判断是没有意义的 每个广播都是一个新的实例
//        if (null != messagePromptDialog) {
//            if (messagePromptDialog.isShowing()) {
//                return;
//            }
//            messagePromptDialog = null;
//        }

        //有可能会引起内存泄漏 临界值
        mMessagePromptDialog = new MessagePromptDialog(ActivityStackControlUtil.currentActivity(),
                "检测到WIFI地址发生变更，请重新登录确保门店收银员信息正确。")
                .setVisibility(true)
                .setOkButton("重新登录", (dialog, which) -> {
                    PreferencesEngine.getInstance().cleanCache();
                    DataSourceManager.clear();
                    PrintManager.clear();
                    ScoComponentProviderManager.clearAfterLoginAssembleStep();
                    ARouter.getInstance().build(ARouterPathConstant.MPOS_LOGIN_ACTIVITY)
                            .navigation(context);
                    ActivityStackControlUtil.popAllActivity();
                });
        mMessagePromptDialog.show();
    }

    private boolean isWifiChange(Context context) {
        String macAddress = DeviceUtil.getNetIp(context);
        if (TextUtils.isEmpty(macAddress)) {
            LogFileUtils.writeLog(ContextUtils.getContext(), "wifi为空");
            return false;
        }
        String lastNetIp = SPUtils.getInstance().getString("lastNetIp");
        SPUtils.getInstance().put("lastNetIp", macAddress);
        if (TextUtils.isEmpty(lastNetIp)) {
            return false;
        }
        LogFileUtils.writeLog(ContextUtils.getContext(), "上次ip地址IP是 = "+lastNetIp + "当前Wi-Fi地址IP是"+macAddress);
        return !macAddress.equals(lastNetIp);
    }

}
