package com.miya.mpos.viewholder;

import android.graphics.Paint;
import android.support.annotation.NonNull;
import android.text.TextUtils;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.miya.iot.sco.beans.viewbeans.ViewGoodsInfo;
import com.miya.iot.sco.core.broadcast.StandardBroadcastInstance;
import com.miya.iot.sco.core_old.base.BaseRecyclerViewAdapter;
import com.miya.iot.sco.core_old.base.BaseRecyclerViewHolder;
import com.miya.iot.sco.core.utils.ContextUtils;
import com.miya.iot.sco.core.utils.StringPriceFormat;
import com.miya.iot.sco.subscriber.ButtonSubscribe;
import com.miya.iot.sco.widget.CenterToasty;
import com.miya.mpos.R;
import com.miya.mpos.widget.dialog.DeleteGoodConfirmDialog;

import java.util.concurrent.TimeUnit;

import rx.Observable;
import rx.functions.Action1;

import static com.miya.iot.sco.constance.BusiConstance.WEIGHT_GOODS;

/**
 * Created by fengyu on 2018/2/5.
 */

public class ScanGoodsViewHolder extends BaseRecyclerViewHolder<ViewGoodsInfo> {

    private Button btnDeleteItem;

    private ImageView ivIncrease;
    private ImageView ivDecrease;
    private TextView tvGoodsNum;

    private TextView tvGoodsName;
    private TextView tvGoodsPrice;
    private TextView tvGoodsOriPrice;
    private TextView tvGoodPromotion;
    private BaseRecyclerViewAdapter adapter;
    private ViewGoodsInfo data;


    public ScanGoodsViewHolder(View itemView, final BaseRecyclerViewAdapter adapter) {
        super(itemView);
        btnDeleteItem = itemView.findViewById(R.id.btnDeleteItem);
        ivIncrease = itemView.findViewById(R.id.ivIncrease);
        ivDecrease = itemView.findViewById(R.id.ivDecrease);
        tvGoodsNum = itemView.findViewById(R.id.tvGoodsNum);
        tvGoodPromotion = itemView.findViewById(R.id.tvGoodPromotion);
        tvGoodsName = itemView.findViewById(R.id.tvGoodsName);

        tvGoodsPrice = itemView.findViewById(R.id.tvGoodsPrice);

        tvGoodsOriPrice = itemView.findViewById(R.id.tvGoodsOriPrice);

        this.adapter = adapter;
        btnDeleteItem.setOnClickListener(v -> {
            if (data != null) {
                DeleteGoodConfirmDialog deleteGoodConfirmDialog = new DeleteGoodConfirmDialog(adapter.getContext());
                deleteGoodConfirmDialog.setSelectPosition(getLayoutPosition()).show();
            }

        });
        Observable.unsafeCreate(new ButtonSubscribe(ivIncrease))
                .throttleFirst(2, TimeUnit.SECONDS)
                .subscribe(view -> {
                    //动态计算优惠  调用接口
                    if (getLayoutPosition() >= 0 && getLayoutPosition() < adapter.getDataSource().size()) {
                        ViewGoodsInfo viewGoodsInfo = (ViewGoodsInfo) adapter.getDataSource().get(getLayoutPosition());
                        if (viewGoodsInfo.isManualPrice()) {
                            CenterToasty.error(adapter.getContext(), "手改价商品不能增加", Toast.LENGTH_LONG).show();
                            return;
                        }
                        StandardBroadcastInstance.sendGoodsIncreaseBroadcast(ContextUtils.getContext(), viewGoodsInfo);
                        checkItemNum(getLayoutPosition());
                    }
                });
        Observable.unsafeCreate(new ButtonSubscribe(ivDecrease))

                .throttleFirst(2, TimeUnit.SECONDS)
                .subscribe(view -> {
                    int num = Integer.parseInt(tvGoodsNum.getText().toString());
                    if (getLayoutPosition() >= 0 && getLayoutPosition() < adapter.getDataSource().size()) {

                        if (num > 1) {
                            StandardBroadcastInstance.sendGoodsDecreaseBroadcast(ContextUtils.getContext(), (ViewGoodsInfo) adapter.getDataSource().get(getLayoutPosition()));
                        }

                        checkItemNum(getLayoutPosition());
                    }
                });
    }


    @Override
    public void setData(@NonNull ViewGoodsInfo data, int position) {
        if (data == null) {
            return;
        }

        this.data = data;

        tvGoodsOriPrice.setTextColor(ContextUtils.getContext().getResources().getColor(R.color.face_pay_text_gay_color));
        tvGoodsName.setTextColor(ContextUtils.getContext().getResources().getColor(R.color.face_pay_text_black_color));

        tvGoodsName.setText(data.getGoodsName());

        tvGoodsPrice.setText("¥" + StringPriceFormat.transStringPriceToDecimalString(data.getGoodsPrice()));
        if (!TextUtils.isEmpty(data.getOriPrice()) && !data.getOriPrice().equals("0") && !data.getOriPrice().equals(data.getGoodsPrice())) {
            tvGoodsOriPrice.setText("¥" + StringPriceFormat.transStringPriceToDecimalString(data.getOriPrice()));
            tvGoodsOriPrice.getPaint().setFlags(Paint.STRIKE_THRU_TEXT_FLAG | Paint.ANTI_ALIAS_FLAG); //下划线
            tvGoodsOriPrice.getPaint().setAntiAlias(true);//抗锯齿
            tvGoodsOriPrice.setVisibility(View.VISIBLE);
        } else {
            tvGoodsOriPrice.setVisibility(View.GONE);
        }
        tvGoodsNum.setText(data.getGoodsNum() + "");

        if(!TextUtils.isEmpty(data.getCouponTypeName())){
            tvGoodPromotion.setVisibility(View.VISIBLE);
            tvGoodPromotion.setText(data.getCouponTypeName());
        }else{
            tvGoodPromotion.setVisibility(View.GONE);
        }

        checkItemNum(position);

    }

    private void checkItemNum(int position) {
        int num = Integer.parseInt(tvGoodsNum.getText().toString());
        if (num > 1) {
//            ivDecrease.setImageResource(R.drawable.btn_reduce_focus_normal);
            ivDecrease.setEnabled(true);
        } else {
//            ivDecrease.setImageResource(R.drawable.btn_reduce_disable_focus);
            ivDecrease.setEnabled(false);
        }
        if (data.getGoodsType() == WEIGHT_GOODS) {
            //称重、计件商品不允许增加
            ivIncrease.setEnabled(false);
//            ivIncrease.setImageResource(R.drawable.btn_increase_disable);
        } else {
            ivIncrease.setEnabled(true);
//            ivIncrease.setImageResource(R.drawable.btn_increase);
        }
    }

}
