package com.miya.mpos.views;

import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.alibaba.android.arouter.facade.annotation.Route;
import com.miya.iot.sco.beans.SelfCashierTerminalConfig;
import com.miya.iot.sco.beans.local.LocalLastPrintInfo;
import com.miya.iot.sco.beans.print.PrinterResult;
import com.miya.iot.sco.beans.viewbeans.ViewStaffInfo;
import com.miya.iot.sco.constance.ARouterPathConstant;
import com.miya.iot.sco.core.actiondispatcher.ViewActions;
import com.miya.iot.sco.core.datasource.DataSourceManager;
import com.miya.iot.sco.core.filepicker.LFilePicker;
import com.miya.iot.sco.core.sharedpreferences.PreferencesEngine;
import com.miya.iot.sco.core.utils.ActivityStackControlUtil;
import com.miya.iot.sco.impl.base.BaseFunCenterActionViewActivity;
import com.miya.iot.sco.impl.component.PrinterComponent;
import com.miya.iot.sco.impl.component.ScoComponentProviderManager;
import com.miya.iot.sco.printer.PrintManager;
import com.miya.iot.sco.widget.CenterToasty;
import com.miya.mpos.R;
import com.miya.mpos.R2;
import com.miya.mpos.widget.dialog.StaffLoginDialog;

import java.util.HashMap;
import java.util.Map;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import rx.android.schedulers.AndroidSchedulers;
import rx.functions.Action1;
import rx.schedulers.Schedulers;

import static com.miya.iot.sco.core.broadcast.BroadcastActionConstance.BROADCAST_STAFF_LOGIN;
import static com.miya.iot.sco.core.component.ScoCoreComponentName.SCO_COMPONENT_PRINTER;
import static com.miya.iot.sco.core.utils.LogFileUtils.CACHE_FILE_PATH;


@Route(path = ARouterPathConstant.MPOS_FUNC_CENTER_ACTIVITY)
public class MposFunctionCenterActivity extends BaseFunCenterActionViewActivity implements View.OnClickListener {

    @BindView(R2.id.ivBack)
    ImageView ivBack;
    @BindView(R2.id.tvBack)
    TextView tvBack;
    @BindView(R2.id.moreOpLookLog)
    LinearLayout moreOpLookLog;
    @BindView(R2.id.moreOpPrintLast)
    LinearLayout moreOpPrintLast;
    @BindView(R2.id.tvExitSystem)
    TextView tvExitSystem;
    @BindView(R2.id.tvLogout)
    TextView tvLogout;
    @BindView(R2.id.tvAccountInfo)
    TextView tvAccountInfo;
    @BindView(R2.id.llStaffSwitch)
    LinearLayout llStaffSwitch;

    private StaffLoginDialog staffLoginDialog;
    private SelfCashierTerminalConfig selfCashierTerminalConfig;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_mpos_function_center);
        ButterKnife.bind(this);
        initView();
    }

    private void initView() {
        functionControl();

        moreOpLookLog.setOnClickListener(this);
        moreOpPrintLast.setOnClickListener(this);
        ivBack.setOnClickListener(this);
        tvBack.setOnClickListener(this);
        tvLogout.setOnClickListener(this);
        tvExitSystem.setOnClickListener(this);
    }

    private void functionControl() {
        //员工登录
        selfCashierTerminalConfig = accountInfoComponent.getSelfCashierTerminalConfig();
        if (null != selfCashierTerminalConfig && selfCashierTerminalConfig.getScoFunctionConfig().isStaffLoginFeatureEnable()) {
            String staffName = selfCashierTerminalConfig.getShopConfig().getStaffName();
            String storeName = selfCashierTerminalConfig.getShopConfig().getStoreName();
            tvAccountInfo.setText(getString(R.string.store_and_staff_info,
                    TextUtils.isEmpty(storeName) ? "--" : storeName, TextUtils.isEmpty(staffName) ? "--" : staffName));
            staffLoginDialog = new StaffLoginDialog(this);
            llStaffSwitch.setOnClickListener(this);
        } else {
            tvAccountInfo.setVisibility(View.GONE);
            llStaffSwitch.setVisibility(View.GONE);
        }
    }


    @Override
    public void onClick(View v) {
        int id = v.getId();
        if (id == R.id.moreOpLookLog) {
            new LFilePicker()
                    .withActivity(this)
                    .withRequestCode(1000)
                    .withStartPath(CACHE_FILE_PATH)
                    .withIsGreater(true)
                    .withFileSize(0)
                    .withMutilyMode(false)
                    .start();
            overridePendingTransition(R.anim.in_right, R.anim.out_left);
        }
        if (id == R.id.moreOpPrintLast) {
            printLastOrder();
        }
        if (id == R.id.ivBack || id == R.id.tvBack) {
            finish();
        }
        if (id == R.id.tvLogout) {
            PreferencesEngine.getInstance().cleanCache();
            DataSourceManager.clear();
            PrintManager.clear();
            ScoComponentProviderManager.clearAfterLoginAssembleStep();
            startActivity(new Intent(MposFunctionCenterActivity.this,
                    MposLoginActivity.class));
            ActivityStackControlUtil.popAllActivity();
        }
        if (id == R.id.llStaffSwitch) {
            staffLoginDialog.show();
        }
        if (id == R.id.tvExitSystem) {
            Intent home = new Intent(Intent.ACTION_MAIN);
            home.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
            home.addCategory(Intent.CATEGORY_HOME);
            startActivity(home);
        }
    }

    @Override
    protected void onStaffLoginSuccess(ViewStaffInfo viewStaffInfo) {
        super.onStaffLoginSuccess(viewStaffInfo);
        if(staffLoginDialog != null && staffLoginDialog.isShowing()){
            staffLoginDialog.dismiss();
        }
        String staffName = viewStaffInfo.getRealName();
        String storeName = selfCashierTerminalConfig.getShopConfig().getStoreName();
        tvAccountInfo.setText(getString(R.string.store_and_staff_info,
                TextUtils.isEmpty(storeName) ? "--" : storeName, TextUtils.isEmpty(staffName) ? "--" : staffName));
        staffLoginDialog = new StaffLoginDialog(this);
    }

    @Override
    protected void onStaffLoginFail(String msg) {
        super.onStaffLoginFail(msg);
        CenterToasty.error(this,TextUtils.isEmpty(msg)?"员工登录失败":msg,Toast.LENGTH_LONG).show();
    }

    @Override
    protected void onScoLocalBroadcastReceive(Intent intent) {
        super.onScoLocalBroadcastReceive(intent);
        if (intent != null && !TextUtils.isEmpty(intent.getAction())) {
            if (BROADCAST_STAFF_LOGIN.equals(intent.getAction())) {
                String userName = (String) intent.getSerializableExtra("userName");
                String password = (String) intent.getSerializableExtra("password");
                staffLogin(userName,password);
            }
        }
    }

    @Override
    protected void onViewActionAbandon(ViewActions viewActions) {
        super.onViewActionAbandon(viewActions);
        CenterToasty.error(this,"操作过快，请间隔一秒再扫描商品",Toast.LENGTH_LONG).show();
    }

}
