package com.miya.mpos.views;

import android.os.Bundle;
import android.os.Handler;
import android.support.annotation.Nullable;
import android.support.constraint.ConstraintLayout;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.alibaba.android.arouter.facade.annotation.Route;
import com.alibaba.android.arouter.launcher.ARouter;
import com.blankj.utilcode.util.LogUtils;
import com.blankj.utilcode.util.NetworkUtils;
import com.blankj.utilcode.util.SPUtils;
import com.miya.iot.sco.beans.local.LocalLoginAccountInfo;
import com.miya.iot.sco.beans.viewbeans.ViewStaffInfo;
import com.miya.iot.sco.constance.ARouterPathConstant;
import com.miya.iot.sco.core.BuildConfig;
import com.miya.iot.sco.core.sharedpreferences.PreferencesEngine;
import com.miya.iot.sco.core.utils.DeviceUtil;
import com.miya.iot.sco.core.utils.LogFileUtils;
import com.miya.iot.sco.impl.base.ScoLoginWithoutConfigCheckActionViewActivity;
import com.miya.iot.sco.update.UpdateManager;
import com.miya.iot.sco.update.bean.UpdateInfoDO;
import com.miya.iot.sco.update.view.UpdateDialog;
import com.miya.iot.sco.widget.CenterToasty;
import com.miya.mpos.R;
import com.miya.mpos.R2;
import com.miya.mpos.widget.dialog.UpgradeDialog;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;


@Route(path = ARouterPathConstant.MPOS_LOGIN_ACTIVITY)
public class MposLoginActivity extends ScoLoginWithoutConfigCheckActionViewActivity {

    @BindView(R2.id.ivMerchantLogo)
    ImageView ivMerchantLogo;
    @BindView(R2.id.etStoteAccInput)
    EditText etStoteAccInput;
    @BindView(R2.id.ivStoteAccInputClear)
    ImageView ivStoteAccInputClear;
    @BindView(R2.id.etStotePwdInput)
    EditText etStotePwdInput;
    @BindView(R2.id.ivStotePwdInputClear)
    ImageView ivStotePwdInputClear;
    @BindView(R2.id.btnStoreLogin)
    TextView btnStoreLogin;
    @BindView(R2.id.cStoreLoginContainer)
    ConstraintLayout cStoreLoginContainer;
    @BindView(R2.id.etStaffAccInput)
    EditText etStaffAccInput;
    @BindView(R2.id.ivStaffAccInputClear)
    ImageView ivStaffAccInputClear;
    @BindView(R2.id.etStaffPwdInput)
    EditText etStaffPwdInput;
    @BindView(R2.id.ivStaffPwdInputClear)
    ImageView ivStaffPwdInputClear;
    @BindView(R2.id.btnStaffLogin)
    TextView btnStaffLogin;
    @BindView(R2.id.cStaffLoginContainer)
    ConstraintLayout cStaffLoginContainer;
    @BindView(R2.id.tvIpAdressInfo)
    TextView tvIpAdressInfo;
    @BindView(R2.id.tvVersionInfo)
    TextView tvVersionInfo;
    @BindView(R2.id.llLoginStep1)
    LinearLayout llLoginStep1;
    @BindView(R2.id.llLoginStep2)
    LinearLayout llLoginStep2;

    UpgradeDialog upgradeDialog;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setFullScreen();
        setContentView(R.layout.activity_new_mpos_login);
        ButterKnife.bind(this);
        initView();
        boolean isWifiChange = checkMacAddress();
        if (!isWifiChange) {
            checkLocalAccountInfo();
        } else {//清理本地登录信息
            PreferencesEngine.getInstance().cleanCache();
        }
    }

    /**
     * wifi mac 限制跨店
     * @return
     */
    private boolean checkMacAddress() {
        String curLocalIp = DeviceUtil.getNetIp(this);
        if (TextUtils.isEmpty(curLocalIp)) {
            return false;
        }
        String lastMacAddress = SPUtils.getInstance().getString("lastNetIp");
        SPUtils.getInstance().put("lastNetIp", curLocalIp);
        if (TextUtils.isEmpty(lastMacAddress)) {
            return false;
        }
        if (curLocalIp.equals(lastMacAddress)) {
            return false;
        }
        CenterToasty.warning(this, getString(R.string.wifi_changed_need_login_again), Toast.LENGTH_LONG).show();
        return true;
    }

    @Override
    public void onWindowFocusChanged(boolean hasFocus) {
        super.onWindowFocusChanged(hasFocus);
        setFullScreen();
    }

    private void initView() {
        tvIpAdressInfo.setText(BuildConfig.BASE_API);
        tvVersionInfo.setText(DeviceUtil.getVersion(this));
        upgradeDialog = new UpgradeDialog(this).setTitle(getString(R.string.find_new_version));

        if (!isStaffLoginFeatureEnable()) {
            btnStoreLogin.setText(getString(R.string.login));
            llLoginStep1.setVisibility(View.GONE);
        } else {
            btnStoreLogin.setText(getString(R.string.next));
        }
    }

    /**
     * 防止账号配置膨胀，可以选择子类继承的方式
     * @return
     */
    protected boolean isStaffLoginFeatureEnable(){
        return true;
    }

    @OnClick({R2.id.btnStoreLogin})
    public void onStoreLoginBtnClicked(View view) {

        String account = etStoteAccInput.getText().toString();
        String passwd = etStotePwdInput.getText().toString();

        if (!TextUtils.isEmpty(account) && !TextUtils.isEmpty(passwd)) {
            showWaitingDialog();
            login(account, passwd, false, false);
        } else {
            CenterToasty.error(this, getString(R.string.incomplete_information)).show();
        }
    }

    @OnClick({R2.id.btnStaffLogin})
    public void onStaffLoginBtnClicked(View view) {
        String account = etStaffAccInput.getText().toString();
        String passwd = etStaffPwdInput.getText().toString();
        if (!TextUtils.isEmpty(account) && !TextUtils.isEmpty(passwd)) {
            staffLogin(account,passwd);
        } else {
            CenterToasty.error(this, getString(R.string.incomplete_information)).show();
        }
    }

    @Override
    protected void onLoginSuccess() {
        super.onLoginSuccess();
        updateVersion();
    }

    @Override
    protected void handlerUpdate(UpdateInfoDO cashUpdateInfoDO) {
        super.handlerUpdate(cashUpdateInfoDO);
        hideWaitingDialog();
        //show upgrade dialog
        try {
            UpdateManager.getInstance().setCustomUpdateDialog(upgradeDialog);
            UpdateManager.getInstance().handlerUpdate(this, cashUpdateInfoDO, null, UpdateDialog.THEME_CUSTOM);
        } catch (Exception e) {
            LogFileUtils.writeLog(this, "handlerUpdate with exception ---->" + e.getMessage() + "\n");
            startSystemInit();
        }
    }

    @Override
    protected void onLoginFailed(String msg, Boolean isScanLogin, Boolean isAutoLogin, Boolean isOfflineLogin) {
        super.onLoginFailed(msg, isScanLogin, isAutoLogin, isOfflineLogin);
        CenterToasty.error(this, msg).show();
        hideWaitingDialog();
    }

    /**
     * 检查本地信息，有存用户名密码 则自动登录
     */
    private void checkLocalAccountInfo() {
        showWaitingDialog();
        LocalLoginAccountInfo loginAccountInfo = PreferencesEngine.getInstance().getLoginAccountInfo();
        if (loginAccountInfo != null) {
            etStoteAccInput.setText(loginAccountInfo.getUserName());
            etStotePwdInput.setText(loginAccountInfo.getPassword());
            String scheme = getIntent().getScheme();
            if ("selfcashier".equals(scheme)) {
                cycleLogin(loginAccountInfo);
            } else {
                login(loginAccountInfo.getUserName(), loginAccountInfo.getPassword(), false, true);
            }
        } else {
            hideWaitingDialog();
        }
    }

    @Override
    protected void onSystemInitSuccess() {
        super.onSystemInitSuccess();
        hideWaitingDialog();
        if(isStaffLoginFeatureEnable()){
            cStaffLoginContainer.setVisibility(View.VISIBLE);
            cStoreLoginContainer.setVisibility(View.GONE);
        }else{
            ARouter.getInstance().build(scoComponentProvider.getEnterActivityArouterPath()).navigation(this);
            this.finish();
        }
    }

    @Override
    protected void onGenerateComponentSuccess() {
        hideWaitingDialog();
        if(isStaffLoginFeatureEnable()){
            cStaffLoginContainer.setVisibility(View.VISIBLE);
            cStoreLoginContainer.setVisibility(View.GONE);
        }else{
            ARouter.getInstance().build(scoComponentProvider.getEnterActivityArouterPath()).navigation(this);
            this.finish();
        }
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            return true;
        }
        return false;
    }

    /**
     * 循环登陆
     * 每隔1秒去判断一次网络 循环60次
     * 防止：开机之后，应用已经自启，但是网络没有准备好
     */
    private void cycleLogin(final LocalLoginAccountInfo loginAccountInfo) {
        new Thread(() -> {
            while (true) {
                if (NetworkUtils.isConnected()) {
                    login(loginAccountInfo.getUserName(), loginAccountInfo.getPassword(), false, true);
                    break;
                } else {
                    try {
                        Thread.sleep(1000);
                    } catch (InterruptedException e) {

                    }
                }
            }
        }).start();
    }
}
