package com.miya.mpos.views.legao;

import android.content.Context;
import android.graphics.Paint;
import android.support.annotation.NonNull;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import com.bumptech.glide.Glide;
import com.miya.iot.sco.beans.viewbeans.ViewGoodsInfo;
import com.miya.iot.sco.core.broadcast.StandardBroadcastInstance;
import com.miya.iot.sco.core.utils.ContextUtils;
import com.miya.iot.sco.core.utils.StringPriceFormat;
import com.miya.iot.sco.core_old.base.BaseRecyclerViewAdapter;
import com.miya.iot.sco.core_old.base.BaseRecyclerViewHolder;
import com.miya.iot.sco.subscriber.ButtonSubscribe;
import com.miya.mpos.R;

import java.util.List;
import java.util.concurrent.TimeUnit;

import rx.Observable;

import static com.miya.iot.sco.constance.BusiConstance.WEIGHT_GOODS;

public class LeGaoGoodsAdapter extends BaseRecyclerViewAdapter<ViewGoodsInfo, BaseRecyclerViewHolder<ViewGoodsInfo>> {


    public LeGaoGoodsAdapter(List<ViewGoodsInfo> dataSource, Context context) {
        super(dataSource, context);
    }

    @Override
    public BaseRecyclerViewHolder<ViewGoodsInfo> onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(getContext()).inflate(R.layout.item_mpos_legao_scan_goods, parent, false);
        LeGaoHolder leGaoHolder = new LeGaoHolder(view);
        return leGaoHolder;
    }

    @Override
    public void onBindViewHolder(BaseRecyclerViewHolder<ViewGoodsInfo> holder, int position) {
        holder.setData(getDataSource().get(position), position);
    }


    public static class LeGaoHolder extends BaseRecyclerViewHolder<ViewGoodsInfo> {

        private TextView tvGoodsBarcode;
        private ImageView ivGoodsLogo;
        private TextView tvGoodsName;
        private TextView tvGoodsPrice;
        private TextView tvGoodsOriPrice;
        private TextView tvGoodsNum;
        private ImageView ivDecrease;
        private ImageView ivIncrease;
        private Button btnDeleteItem;
        private ViewGoodsInfo data;

        public LeGaoHolder(View itemView) {
            super(itemView);
            tvGoodsBarcode = itemView.findViewById(R.id.tvGoodsBarcode);
            ivGoodsLogo = itemView.findViewById(R.id.ivGoodsLogo);
            tvGoodsName = itemView.findViewById(R.id.tvGoodsName);
            tvGoodsPrice = itemView.findViewById(R.id.tvGoodsPrice);
            tvGoodsOriPrice = itemView.findViewById(R.id.tvGoodsOriPrice);
            tvGoodsNum = itemView.findViewById(R.id.tvGoodsNum);
            ivDecrease = itemView.findViewById(R.id.ivDecrease);
            ivIncrease = itemView.findViewById(R.id.ivIncrease);
            btnDeleteItem = itemView.findViewById(R.id.btnDeleteItem);

            Observable.unsafeCreate(new ButtonSubscribe(ivIncrease))
                    .throttleFirst(2, TimeUnit.SECONDS)
                    .subscribe(view -> {
                        if (data == null) {
                            return;
                        }
                        if (getLayoutPosition() >= 0 && getLayoutPosition() == getAdapterPosition()) {//没有被刷新影响
                            ViewGoodsInfo viewGoodsInfo = data;
                            StandardBroadcastInstance.sendGoodsIncreaseBroadcast(ContextUtils.getContext(), viewGoodsInfo);
                            checkItemNum();
                        }
                    });

            Observable.unsafeCreate(new ButtonSubscribe(ivDecrease))
                    .throttleFirst(2, TimeUnit.SECONDS)
                    .subscribe(view -> {
                        if (data == null) {
                            return;
                        }
                        int num = Integer.parseInt(tvGoodsNum.getText().toString());
                        if (getLayoutPosition() >= 0 && getLayoutPosition() == getAdapterPosition()) {

                            if (num > 1) {
                                StandardBroadcastInstance.sendGoodsDecreaseBroadcast(ContextUtils.getContext(), data);
                            }
                            checkItemNum();
                        }
                    });
            Observable.unsafeCreate(new ButtonSubscribe(btnDeleteItem))
                    .throttleFirst(2, TimeUnit.SECONDS)
                    .subscribe(view -> {
                        if (data == null) {
                            return;
                        }
                        if (getLayoutPosition() >= 0 && getLayoutPosition() == getAdapterPosition()) {
                            StandardBroadcastInstance.sendGoodsDeleteBroadcast(ContextUtils.getContext(), getAdapterPosition());
                        }
                    });
        }

        @Override
        public void setData(@NonNull ViewGoodsInfo data, int position) {
            this.data = data;
            tvGoodsBarcode.setText(data.getBarcode());
            Glide.with(itemView.getContext()).load(data.getGoodsImgUrl()).into(ivGoodsLogo);
            tvGoodsName.setText(data.getGoodsName());
            tvGoodsPrice.setText("¥" + StringPriceFormat.transStringPriceToDecimalString(data.getGoodsPrice()));
            if (!TextUtils.isEmpty(data.getOriPrice()) && !data.getOriPrice().equals("0") && !data.getOriPrice().equals(data.getGoodsPrice())) {
                tvGoodsOriPrice.setText("¥" + StringPriceFormat.transStringPriceToDecimalString(data.getOriPrice()));
                tvGoodsOriPrice.getPaint().setFlags(Paint.STRIKE_THRU_TEXT_FLAG | Paint.ANTI_ALIAS_FLAG); //下划线
                tvGoodsOriPrice.getPaint().setAntiAlias(true);//抗锯齿
                tvGoodsOriPrice.setVisibility(View.VISIBLE);
            } else {
                tvGoodsOriPrice.setVisibility(View.GONE);
            }
            tvGoodsNum.setText(data.getGoodsNum() + "");
            checkItemNum();

        }

        private void checkItemNum() {
            int num = Integer.parseInt(tvGoodsNum.getText().toString());
            if (num > 1) {
                ivDecrease.setEnabled(true);
            } else {
                ivDecrease.setEnabled(false);
            }
        }
    }

}