package com.miya.mpos.views.legao;

import android.os.Bundle;
import android.support.annotation.Nullable;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.View;
import android.widget.TextView;

import com.alibaba.android.arouter.facade.annotation.Route;
import com.alibaba.android.arouter.launcher.ARouter;
import com.miya.iot.sco.beans.SelfCashierTerminalConfig;
import com.miya.iot.sco.beans.viewbeans.ViewLocalPreparePaymentInfo;
import com.miya.iot.sco.beans.viewbeans.ViewMemberInfo;
import com.miya.iot.sco.beans.viewbeans.ViewPaySuccessInfo;
import com.miya.iot.sco.beans.viewbeans.ViewScanGoodsPageInfo;
import com.miya.iot.sco.constance.ARouterPathConstant;
import com.miya.iot.sco.core.broadcast.StandardBroadcastInstance;
import com.miya.iot.sco.core.utils.ContextUtils;
import com.miya.iot.sco.core.utils.KeywordUtil;
import com.miya.iot.sco.core.utils.ScanGunKeyEventHelper;
import com.miya.iot.sco.core.utils.StringPriceFormat;
import com.miya.iot.sco.impl.base.ScoBalanceOrderActionViewActivity;
import com.miya.iot.sco.net.ErrorPowerEnum;
import com.miya.iot.sco.widget.CenterToasty;
import com.miya.mpos.R;
import com.miya.mpos.R2;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;

import static com.miya.iot.sco.constance.BusiConstance.VIEW_PAYS_STATE_SUCCESS;
import static com.sdy.miya.moblie.component.pay.core.constance.PayWayConstance.BARCODE_TYPE;


@Route(path = ARouterPathConstant.LEGAO_MPOS_BALANCE_ORDER_ACTIVITY)
public class MposLeGaoBalanceOrderActivity extends ScoBalanceOrderActionViewActivity {

    ViewScanGoodsPageInfo viewScanGoodsPageInfo;
    ViewMemberInfo viewMemberInfo;

    @BindView(R2.id.layoutMember)
    View layoutMember;

    @BindView(R2.id.tvMemberName)
    TextView tvMemberName;

    @BindView(R2.id.tvPayPrice)
    TextView tvPayPrice;

    @BindView(R2.id.tvCouponPrice)
    TextView tvCouponPrice;

    @BindView(R2.id.tvOrderPrice)
    TextView tvOrderPrice;

    @BindView(R2.id.tvAccountInfo)
    TextView tvAccountInfo;

    @BindView(R2.id.tvPay)
    TextView tvPay;

    @BindView(R2.id.tvGoodsNum)
    TextView tvGoodsNum;

    @BindView(R2.id.layoutBalance)
    View layoutBalance;

    @BindView(R2.id.layoutPayWait)
    View layoutPayWait;

    ScanGunKeyEventHelper scanGunKeyEventHelper;

    /**
     * 标示是否正在支付中
     */
    boolean isPaying;

    SelfCashierTerminalConfig selfCashierTerminalConfig;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setFullScreen();
        setContentView(R.layout.activity_new_mpos_balance_order);
        ButterKnife.bind(this);
        initView();
    }

    @Override
    public void onWindowFocusChanged(boolean hasFocus) {
        super.onWindowFocusChanged(hasFocus);
        setFullScreen();
    }

    private void initView() {
        if (getIntent() == null || getIntent().getExtras() == null) {
            CenterToasty.error(ContextUtils.getContext(), "数据错误").show();
            finish();
        }
        if (accountInfoComponent != null) {
            selfCashierTerminalConfig = accountInfoComponent.getSelfCashierTerminalConfig();
            if(selfCashierTerminalConfig.getScoFunctionConfig().isStaffLoginFeatureEnable()){
                String staffName = selfCashierTerminalConfig.getShopConfig().getStaffName();
                String storeName = selfCashierTerminalConfig.getShopConfig().getStoreName();
                tvAccountInfo.setText(getString(R.string.store_and_staff_info,
                        TextUtils.isEmpty(storeName)?"--":storeName,TextUtils.isEmpty(staffName)?"--":staffName ));
                tvAccountInfo.setVisibility(View.VISIBLE);
            }else{
                tvAccountInfo.setVisibility(View.GONE);
            }
        }

        viewScanGoodsPageInfo = (ViewScanGoodsPageInfo) getIntent().getExtras().getSerializable("viewScanGoodsPageInfo");
        viewMemberInfo = (ViewMemberInfo) getIntent().getExtras().getSerializable("viewMemberInfo");

        if (viewMemberInfo != null) {
            layoutMember.setVisibility(View.VISIBLE);
            tvMemberName.setText(viewMemberInfo.getMemberName());
        } else {
            layoutMember.setVisibility(View.GONE);
        }

        //set price
        if(viewScanGoodsPageInfo.getOrderInfo() != null){
            tvOrderPrice.setText("￥" + StringPriceFormat.transStringPriceToDecimalString(viewScanGoodsPageInfo.getOrderInfo().getOrderPrice()));
            if (!TextUtils.isEmpty(viewScanGoodsPageInfo.getOrderInfo().getCouponPrice())) {
                tvCouponPrice.setText("-￥" + StringPriceFormat.transStringPriceToDecimalString(viewScanGoodsPageInfo.getOrderInfo().getCouponPrice()));
            } else {
                tvCouponPrice.setText("-￥0.00");
            }
            tvPayPrice.setText("￥" + StringPriceFormat.transStringPriceToDecimalString(viewScanGoodsPageInfo.getOrderInfo().getPayPrice()));

            tvGoodsNum.setText(KeywordUtil.matcherSearch(getResources().getColor(R.color.face_pay_text_orange_color), getString(R.string.total_amount, (viewScanGoodsPageInfo.getOrderInfo().getGoodsCount() + "")), "[0-9]"));
        }


        //init scan gun
        scanGunKeyEventHelper = new ScanGunKeyEventHelper();
        scanGunKeyEventHelper.setOnBarCodeCatchListener(new ScanGunKeyEventHelper.OnScanSuccessListener() {
            @Override
            public void onScanSuccess(String barcode) {
                if (isPaying) {
                    return;
                }
                go2Pay(barcode);
            }
        });
    }

    private void go2Pay(String payCode) {
        //go 2 pay
        if (TextUtils.isEmpty(payCode)) {
            return;
        }
        isPaying = true;
        startPay(viewScanGoodsPageInfo, payCode, BARCODE_TYPE);
    }

    @Override
    protected void onPaySuccess(ViewPaySuccessInfo viewPaySuccessInfo, boolean isUploadSuccess, String payType) {
        super.onPaySuccess(viewPaySuccessInfo, isUploadSuccess, payType);
        isPaying = false;
        hideWaitingDialog();
        //start 2 pay result activity
        go2PayResultActivity(viewPaySuccessInfo, isUploadSuccess, payType);
    }

    private void go2PayResultActivity(ViewPaySuccessInfo viewPaySuccessInfo, boolean isUploadSuccess, String payType) {
        Bundle bundle = new Bundle();
        bundle.putSerializable("viewPaySuccessInfo", viewPaySuccessInfo);
        bundle.putBoolean("isUploadSuccess", isUploadSuccess);
        bundle.putString("payType", payType);
        ViewLocalPreparePaymentInfo landPaymentInfo = new ViewLocalPreparePaymentInfo();
        landPaymentInfo.setPayType(payType);
        landPaymentInfo.setPrintFooterMessage(selfCashierTerminalConfig.getPrintConfig().getPrintFooterMessage());
        landPaymentInfo.setPayState(VIEW_PAYS_STATE_SUCCESS);
        viewPaySuccessInfo.setStoreName(selfCashierTerminalConfig.getHuihuaConfig().getStoreName());
        landPaymentInfo.setGoodsNum(viewScanGoodsPageInfo.getGoodsInfoList().size());
        landPaymentInfo.setGoodsList(viewScanGoodsPageInfo.getGoodsInfoList());
        if(null != viewMemberInfo){
            landPaymentInfo.setViewMemberInfo(viewMemberInfo);
            landPaymentInfo.setMemberInputCode(viewMemberInfo.getMemberNo());
        }
        landPaymentInfo.setLandScanGoodsPageInfo(viewScanGoodsPageInfo);
        landPaymentInfo.setPayServiceResponse(viewPaySuccessInfo.getPayServiceResponse());
        try {
            landPaymentInfo.setCouponPayPrice(Integer.parseInt(viewScanGoodsPageInfo.getOrderInfo().getCouponPrice()));
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        bundle.putSerializable("viewLocalPreparePaymentInfo", landPaymentInfo);
        try {
            ARouter.getInstance().build(selfCashierTerminalConfig.getScoRuntimeConfig().getScoPayResultActionViewActivityRouterPath()).with(bundle).navigation(this);
            return;
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        ARouter.getInstance().build(ARouterPathConstant.MPOS_PAY_RESULT_ACTIVITY).with(bundle).navigation(this);
    }

    @Override
    protected void onPayFailed(String errorMsg, String payType) {
        super.onPayFailed(errorMsg, payType);
        isPaying = false;
        hideWaitingDialog();
        showErrMsg(errorMsg);
    }

    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        StandardBroadcastInstance.sendUpdateOperateTimeBroadcast(this);
        scanGunKeyEventHelper.analysisKeyEvent(event);
        return true;
    }

    @OnClick(R2.id.layoutBack)
    public void onBackClicked(View view) {
        finish();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (scanGunKeyEventHelper != null) {
            scanGunKeyEventHelper.onDestroy();
        }
    }

    private void showErrMsg(String errorMsg) {
        if (TextUtils.isEmpty(errorMsg)) {
            CenterToasty.error(this, "未知错误").show();
            return;
        }
        if (errorMsg.contains(ErrorPowerEnum.ERR_POWER_HOME.getCode())) {
            showErrorDialog(errorMsg, errorMsg, dialog -> {
                hideErrorDialog();
                clearDataSource();
                go2Home();
            });
            return;
        }
        if (errorMsg.contains(ErrorPowerEnum.ERR_POWER_NONE.getCode())) {
            showErrorDialog(errorMsg, errorMsg);
            return;
        }
        CenterToasty.error(this,errorMsg).show();
    }

    public void go2Home() {
        try {
            ARouter.getInstance().build(selfCashierTerminalConfig.getScoRuntimeConfig().getScoHomeActionViewActivityRouterPath()).navigation(this);
            finish();
            return;
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        ARouter.getInstance().build(ARouterPathConstant.MPOS_HOME_ACTIVITY).navigation(this);
    }


    @Override
    protected void showWaitingDialog() {
        super.showWaitingDialog();
    }
    @Override
    protected void hideWaitingDialog() {
        super.hideWaitingDialog();
    }
}
