package com.miya.mpos.widget.dialog;

import android.animation.ObjectAnimator;
import android.content.Context;
import android.support.annotation.NonNull;
import android.text.TextUtils;
import android.view.Gravity;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.view.animation.DecelerateInterpolator;
import android.widget.ImageView;
import android.widget.TextView;

import com.blankj.utilcode.util.ScreenUtils;
import com.miya.iot.sco.core.utils.ContextUtils;
import com.miya.iot.sco.core.utils.StringPriceFormat;
import com.miya.iot.sco.subscriber.ButtonSubscribe;
import com.miya.iot.sco.widget.CenterToasty;
import com.miya.mpos.R;
import com.miya.mpos.widget.ClearTextView;
import com.miya.mpos.widget.DecimalNumberKeyboard;


import java.util.concurrent.TimeUnit;

import rx.Observable;
import rx.functions.Action1;


public class MposInputPriceDialog extends BaseDialog {

    /**
     * 确定按钮点击
     */
    public final static int ACTION_CONFIRM = 0x01;

    /**
     * 关闭按钮点击
     */
    public final static int ACTION_CLOSE = 0x02;


    /**
     * 输入内容（只允许含一个小数点，只能有两位小数)
     */
    ClearTextView tvInputContent;

    /**
     * 标题（商品名称）
     */
    TextView tvTitle;

    /**
     * 关闭按钮
     */
    ImageView ivClose;

    /**
     * 数字键盘
     */
    DecimalNumberKeyboard numKeyboard;

    /**
     * 输入内容（只允许含一个小数点，只能有两位小数)
     */
    String inputContent = "";

    /**
     * 确定按钮
     */
    TextView btnConfirm;

    /**
     * 错误信息
     */
    TextView tvErrMsg;


    public MposInputPriceDialog(@NonNull Context context) {
        super(context);
        initWindow();
        setContentView(R.layout.dialog_manual_input_price);
        setCancelable(false);
        setCanceledOnTouchOutside(false);
        //findView
        tvInputContent = findViewById(R.id.tvInputCode);
        tvTitle = findViewById(R.id.tvTitle);
        ivClose = findViewById(R.id.ivClose);
        numKeyboard = findViewById(R.id.numberKeyBoard);
        tvErrMsg = findViewById(R.id.tvErrMsg);
        btnConfirm = findViewById(R.id.btnConfirm);
        Observable.unsafeCreate(new ButtonSubscribe(btnConfirm))
                .throttleFirst(1500, TimeUnit.MILLISECONDS)
                .subscribe(new Action1<View>() {
                    @Override
                    public void call(View view) {
                        if (tvInputContent != null) {
                            if (TextUtils.isEmpty(tvInputContent.getText())) {
                                startErrShakeAnim();
                            } else if (inputContent.length() > 7) {
                                //输入价格不能超过9999.99元
                                CenterToasty.error(ContextUtils.getContext(), getContext().getResources().getString(R.string.str_above_1w)).show();
                                startErrShakeAnim();
                            } else if (Integer.parseInt(StringPriceFormat.transStringYuan2Fen(inputContent)) == 0) {
                                //价格为0报错提示
                                CenterToasty.error(ContextUtils.getContext(), getContext().getResources().getString(R.string.str_input_not_allow_zero)).show();
                                startErrShakeAnim();
                            } else if (Integer.parseInt(StringPriceFormat.transStringYuan2Fen(inputContent)) > 999999) {
                                //输入价格不能超过9999.99元
                                CenterToasty.error(ContextUtils.getContext(), getContext().getResources().getString(R.string.str_above_1w)).show();
                                startErrShakeAnim();
                            } else {
                                if (listener != null) {
                                    if (inputContent.endsWith(".")) {
                                        listener.onActionDone(ACTION_CONFIRM, inputContent + "00");
                                    } else {
                                        listener.onActionDone(ACTION_CONFIRM, inputContent);
                                    }
                                }
                            }
                        }
                    }
                });
        ivClose.setOnClickListener(v -> {
            if (listener != null) {
                listener.onActionDone(ACTION_CLOSE, "");
            }
        });

        numKeyboard.setKeyboardClickListener(num -> {
            if (num == DecimalNumberKeyboard.DOT) {
                if (!inputContent.contains(".") && !TextUtils.isEmpty(inputContent)) {
                    inputContent += ".";
                }
            } else if (num == DecimalNumberKeyboard.DELETE) {
                //删除
                if (inputContent.length() >= 2) {
                    inputContent = inputContent.substring(0, inputContent.length() - 1);
                } else {
                    inputContent = "";
                }
            } else {
                if (inputContent.contains(".")) {
                    String tmp = inputContent.substring(inputContent.indexOf("."), inputContent.length() - 1);
                    if (tmp.length() < 2) {
                        inputContent += num;
                    }
                } else {
                    if (inputContent.startsWith("0") && inputContent.length() == 1 && num == 0) {
                        //第二位不能输入0,无动作

                    } else {
                        inputContent += num;
                    }
                }
            }
            tvInputContent.setText(inputContent);
            if (TextUtils.isEmpty(inputContent)) {
                //开始晃动动画
                startErrShakeAnim();
                btnConfirm.setEnabled(false);
            }else{
                btnConfirm.setEnabled(true);
            }
        });

        tvInputContent.setOnClearListener(new ClearTextView.OnClearListener() {
            @Override
            public void onClearClick() {
                inputContent = "";
            }
        });
    }

    private void initWindow() {
        Window win = this.getWindow();
        win.getDecorView().setPadding(0, 0, 0, 0);
        WindowManager.LayoutParams lp = win.getAttributes();
        lp.width = WindowManager.LayoutParams.MATCH_PARENT;
        lp.height = ScreenUtils.getScreenHeight() * 880/1280;
        lp.gravity = Gravity.BOTTOM;
        lp.windowAnimations = R.style.BottomInAndOutStyle;
        win.setAttributes(lp);
        win.setBackgroundDrawableResource(android.R.color.transparent);
    }

    /**
     * 设置商品名称或标题
     * @param title
     * @return
     */
    public MposInputPriceDialog setTitle(String title) {
        if (tvTitle != null && !TextUtils.isEmpty(title)) {
            tvTitle.setText(title);
        }
        return this;
    }

    public MposInputPriceDialog setErrMsg(String errMsg) {
        if (!TextUtils.isEmpty(errMsg)) {
            tvErrMsg.setText(errMsg);
            tvErrMsg.setVisibility(View.VISIBLE);
        }
        return this;
    }

    @Override
    public void dismiss() {
        if (tvInputContent != null) {
            tvInputContent.setText("");
            inputContent = "";
            tvErrMsg.setText("");
            tvErrMsg.setVisibility(View.INVISIBLE);
        }
        super.dismiss();
    }

    /**
     * 开始错误晃动动画
     */
    public void startErrShakeAnim() {
        if (tvInputContent != null) {
            ObjectAnimator shakeAnimator = ObjectAnimator.ofFloat(tvInputContent, "translationX", 0, 15, 0, -15, 0);
            shakeAnimator.setDuration(200);
            shakeAnimator.setInterpolator(new DecelerateInterpolator());
            shakeAnimator.setRepeatCount(1);
            shakeAnimator.start();
        }
    }

    /**
     * 设置业务动作监听
     *
     * @param listener 监听
     * @return
     */
    public MposInputPriceDialog setOnActionDoneListener(OnActionDoneListener listener) {
        this.listener = listener;
        return this;
    }

    /**
     * 业务动作监听
     */
    OnActionDoneListener listener;

    /**
     * 业务动作完成时的监听接口
     */
    public interface OnActionDoneListener {
        /**
         * 动作结束回调
         *
         * @param action 动作
         * @param result 结果
         */
        void onActionDone(int action, String result);
    }
}
