package com.miya.mpos.widget.dialog;

import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.drawable.Drawable;
import android.os.Build;
import android.support.annotation.NonNull;
import android.text.TextUtils;
import android.view.Gravity;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.jakewharton.rxbinding.view.RxView;
import com.miya.iot.sco.update.view.UpdateDialog;
import com.miya.mpos.R;

import java.util.concurrent.TimeUnit;

import rx.functions.Action1;

/**
 * Created by pupu on 2019-07-03
 * 佛祖开光，永无bug
 * God bless U
 */
public class UpgradeDialog extends UpdateDialog {

    private int mState = STATE_IDLE;

    TextView tvTitle, tvContent, tvOk;

    View vTitle;

    ProgressBar progressBar;

    private UpdateDialogClickListener mUpdateDialogClickListener;

    public UpgradeDialog(Context context) {
        super(context);

        initWindow();
        setFullScreen();
        setContentView(R.layout.dialog_upgrade);

        mState = STATE_IDLE;

        //findView
        tvTitle = findViewById(R.id.tvTitle);
        tvContent = findViewById(R.id.tvContent);
        tvOk = findViewById(R.id.tvOk);
        vTitle = findViewById(R.id.vTitle);
        progressBar = findViewById(R.id.pbProgress);

        RxView.clicks(tvOk)
                .throttleFirst(1500, TimeUnit.MILLISECONDS)
                .subscribe(new Action1<Void>() {
                    @Override
                    public void call(Void aVoid) {
                        if (mUpdateDialogClickListener != null) {
                            if (mState == STATE_DOWNLOAD_FAILED) {
                                mUpdateDialogClickListener.onRetryClick();
                            } else {
                                mUpdateDialogClickListener.onStartClick();
                            }
                        }
                    }
                }, new Action1<Throwable>() {
                    @Override
                    public void call(Throwable throwable) {
                        throwable.printStackTrace();
                    }
                });
    }

    @Override
    public UpdateDialog setMessage(String msg) {
        if (!TextUtils.isEmpty(msg)) {
            tvContent.setText(msg);
        }
        return this;
    }

    @Override
    public UpdateDialog setProgress(int progress) {
        if (mState == STATE_DOWNLOADING) {
            tvTitle.setText("下载中...");
            tvContent.setText(progress + "%");
            progressBar.setProgress(progress);
        }
        return this;
    }

    @Override
    public UpdateDialog setState(int state) {
        mState = state;
        if (state == STATE_IDLE) {
            progressBar.setVisibility(View.GONE);
            tvOk.setVisibility(View.VISIBLE);
        } else if (state == STATE_DOWNLOADING) {
            progressBar.setVisibility(View.VISIBLE);
            progressBar.setMax(100);
            int[] attrs = new int[]{android.R.attr.background};
            TypedArray typedArray = getContext().obtainStyledAttributes(R.style.upgradeTitleImg, attrs);
            Drawable drawable = typedArray.getDrawable(0);
            vTitle.setBackground(drawable);
            typedArray.recycle();
            tvOk.setVisibility(View.INVISIBLE);
        } else if (state == STATE_DOWNLOAD_FAILED) {
            //在下载失败后, 是否提供取消按钮? 目前固定提供取消按钮.
            tvOk.setText("重试");
            int[] attrs = new int[]{android.R.attr.background};
            TypedArray typedArray = getContext().obtainStyledAttributes(R.style.upgradeTitleImgFailed, attrs);
            Drawable drawable = typedArray.getDrawable(0);
            vTitle.setBackground(drawable);
            typedArray.recycle();
            tvContent.setText("下载失败");
            tvOk.setVisibility(View.VISIBLE);
        } else if (state == STATE_INSTALLING) {
            tvContent.setText("正在安装");
            tvOk.setVisibility(View.INVISIBLE);
        } else if (state == STATE_DOWNLOAD_SUCCESS) {
            tvContent.setText("安装完成");
        }
        return this;
    }

    @Override
    public UpdateDialog setHasCancelBtn(boolean hasCancelBtn) {
        return null;
    }

    @Override
    public UpdateDialog setUpdateDialogTheme(int theme) {
        return null;
    }

    @Override
    public UpdateDialog setUpdateDialogClickListener(@NonNull UpdateDialogClickListener listener) {
        mUpdateDialogClickListener = listener;
        return this;
    }

    private void initWindow() {
        Window win = this.getWindow();
        win.getDecorView().setPadding(0, 0, 0, 0);
        WindowManager.LayoutParams lp = win.getAttributes();
        lp.width = WindowManager.LayoutParams.MATCH_PARENT;
        lp.height = WindowManager.LayoutParams.WRAP_CONTENT;
        lp.gravity = Gravity.CENTER;
        lp.windowAnimations = R.style.BottomInAndOutStyle;
        win.setAttributes(lp);
        win.setBackgroundDrawableResource(android.R.color.transparent);
    }

    public UpgradeDialog setTitle(String title) {
        if (!TextUtils.isEmpty(title)) {
            tvTitle.setText(title);
        }
        return this;
    }

    public UpgradeDialog setContent(String content) {
        if (!TextUtils.isEmpty(content)) {
            tvContent.setText(content);
        }
        return this;
    }

    /**
     * 隐藏虚拟按键，并且全屏
     */
    protected void setFullScreen() {
        //隐藏虚拟按键，并且全屏
        if (Build.VERSION.SDK_INT > 11 && Build.VERSION.SDK_INT < 19) { // lower api
            View v = this.getWindow().getDecorView();
            v.setSystemUiVisibility(View.GONE);
        } else if (Build.VERSION.SDK_INT >= 19) {
            //for new api versions.
            View decorView = getWindow().getDecorView();
            int uiOptions = View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                    | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                    | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                    | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION // hide nav bar
                    | View.SYSTEM_UI_FLAG_FULLSCREEN // hide status bar
                    | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY;
            decorView.setSystemUiVisibility(uiOptions);
            getWindow().addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_NAVIGATION);
        }
    }

    @Override
    public void onWindowFocusChanged(boolean hasFocus) {
        super.onWindowFocusChanged(hasFocus);
        setFullScreen();
    }
}
