package com.sdy.huihua.selfcashier_terminal;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Bundle;
import android.os.Environment;
import android.os.Process;
import android.support.annotation.NonNull;
import android.support.multidex.MultiDexApplication;
import android.util.DisplayMetrics;

import com.alibaba.android.arouter.launcher.ARouter;
import com.blankj.utilcode.util.AppUtils;
import com.blankj.utilcode.util.CrashUtils;
import com.kwai.koom.javaoom.KOOM;
import com.miya.iot.sco.constance.BusiConstance;
import com.miya.iot.sco.core.exception.CrashHandler;
import com.miya.iot.sco.core.sharedpreferences.PreferencesEngine;
import com.miya.iot.sco.core.utils.ContextUtils;
import com.miya.iot.sco.core.utils.D;
import com.miya.iot.sco.core.utils.DateUtils;
import com.miya.iot.sco.core.utils.LogFileUtils;
import com.miya.iot.sco.core.utils.OsUtils;
import com.miya.iot.sco.core.utils.VoiceManger;
import com.miya.iot.sco.core_old.utils.TTSHelper;
import com.miya.iot.sco.datasource.helper.WanningHelper;
import com.miya.iot.sco.impl.component.ScoComponentProviderManager;
import com.miya.iot.sco.net.ApiClient;
import com.miya.print.PrinterManager;
import com.sdy.miya.moblie.component.pay.core.net.MiYaPayMobileApiClient;
import com.squareup.leakcanary.LeakCanary;
import com.squareup.leakcanary.RefWatcher;
import com.tencent.bugly.crashreport.CrashReport;

import java.io.File;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Map;

import okhttp3.logging.HttpLoggingInterceptor;


public class SelfCashierTerminalApplication extends MultiDexApplication {
    /**
     * 标准版
     */
    private static final String PROCESS_STANDARD_POS = "com.miyapay.huihua.verification";
    /**
     * 微信 POS
     */
    private static final String PROCESS_WEIXIN_POS = "com.miyapay.huihua.verification.weixin";
    /**
     * 标准MPOS
     */
    private static final String PROCESS_STANDARD_MPOS = "com.miyapay.mpos";
    /**
     * 万宁MPOS
     */
    private static final String PROCESS_WANNING_MPOS = "com.miyapay.wnmpos";
    /**
     * 乐高MPOS
     */
    private static final String PROCESS_LEGAO_MPOS = "com.miyapay.legaompos";
    /**
     * 横版POS
     */
    private static final String PROCESS_STANDARD_LANDPOS = "com.miyapay.pos";

    private static SelfCashierTerminalApplication mApp;
    private RefWatcher refWatcher;

    @Override
    public void onCreate() {
        super.onCreate();

        if (LeakCanary.isInAnalyzerProcess(this)) {
            return;
        }

        if (BuildConfig.DEBUG) {
            LeakCanary.install(this);
        }

        mApp = this;
        String processName = OsUtils.getProcessName(this,
                Process.myPid());


        if (processName != null) {
            boolean defaultProcess = (processName
                    .equals(PROCESS_STANDARD_POS)
                    || processName
                    .equals(PROCESS_WEIXIN_POS)
                    || processName
                    .equals(PROCESS_STANDARD_MPOS)
                    || processName
                    .equals(PROCESS_WANNING_MPOS)
                    || processName
                    .equals(PROCESS_LEGAO_MPOS)
                    || processName
                    .equals(PROCESS_STANDARD_LANDPOS));
            if (defaultProcess) {
                if (!BuildConfig.DEBUG) {
                    initBugly(processName);
                }
                initArouter();
                ContextUtils.init(this);
                PreferencesEngine.init(this);
                //语音播报初始化
                initVoiceComponent();
                // 初始化自定义异常handler
                String tmpDir = Environment.getExternalStorageDirectory().toString() + File.separator + "/miyaterminal/log/crashLog/";
                CrashUtils.init(tmpDir);
                //初始化打印sdk
                PrinterManager.getInstance().init(this);
                //自助收银组件初始化
                ScoComponentProviderManager.init(this);
                HttpLoggingInterceptor loggingInterceptor = null;
                try {
                    loggingInterceptor = new HttpLoggingInterceptor(new HttpLoggingInterceptor.Logger() {
                        @Override
                        public void log(String message) {
                            if (!LogFileUtils.isProhibitWrite) {
                                LogFileUtils.writeLog(ContextUtils.getContext(), DateUtils.getDateStringByTimeStamp(System.currentTimeMillis()) + " " + message + "\n");
                            }
                            if (com.miya.iot.sco.core.BuildConfig.DEBUG) {
                                D.d(ApiClient.TAG, message);
                            }
                        }
                    });
                    loggingInterceptor.setLevel(HttpLoggingInterceptor.Level.BODY);
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
                MiYaPayMobileApiClient.init(true, loggingInterceptor);
                //增加各个生命周期的监听
                registerActivityLifecycleCallbacks(mCallbacks);
                //oom注入
                initOOM();
            }
        }

    }

    /**
     * 初始化oom
     */
    private void initOOM() {
        String oomPath = Environment
                .getExternalStorageDirectory().getPath() +
                File.separator +
                "miyaterminal" +
                File.separator +
                "oom" +
                File.separator;

        KOOM.init(this);

        File logFiles = new File(oomPath);
        if (!logFiles.exists()) {
            logFiles.mkdirs();
        }
        KOOM.getInstance().setRootDir(oomPath);
    }

    @Override
    public void onTerminate() {
        super.onTerminate();
        unregisterActivityLifecycleCallbacks(mCallbacks);
    }

    /**
     * 语音组件初始化
     */
    private void initVoiceComponent( ) {
       VoiceManger.initManager(BusiConstance.WEIXIN, VoiceManger.Scene.CUSTOM_SCENE.setDesc("声音组件初始化"), "");
    }

    /**
     * 初始化路由
     */
    private void initArouter() {
        if (AppUtils.isAppDebug()) {
            ARouter.openLog();     // 打印日志
            ARouter.openDebug();   // 开启调试模式(如果在InstantRun模式下运行，必须开启调试模式！线上版本需要关闭,否则有安全风险)
            ARouter.printStackTrace();
        }
        ARouter.init(this);
    }

    private void initBugly(String proccessName) {
        //发生异常的时候的回调
        CrashReport.UserStrategy strategy = new CrashReport.UserStrategy(this);
        strategy.setCrashHandleCallback(new CrashReport.CrashHandleCallback() {
            public Map<String, String> onCrashHandleStart(int crashType, String errorType,
                                                          String errorMessage, String errorStack) {
                LinkedHashMap<String, String> map = new LinkedHashMap<String, String>();
                LogFileUtils.writeLog(ContextUtils.getContext(),
                        DateUtils.getDateStringByTimeStamp(System.currentTimeMillis()) +
                                "####errorType####" + errorType + "\n" +
                                "####errorMessage####" + errorMessage + "\n" +
                                "####errorStack####" + errorStack);
                return map;
            }
            @Override
            public byte[] onCrashHandleStart2GetExtraDatas(int crashType, String errorType,
                                                           String errorMessage, String errorStack) {
                return null;
            }

        });
        strategy.setAppVersion(AppUtils.getAppVersionName()); //App的版本
        strategy.setAppPackageName(AppUtils.getAppPackageName()); //App的包名
        //万宁mpos
        if(PROCESS_WANNING_MPOS.equals(proccessName)){
            CrashReport.initCrashReport(getApplicationContext(), "c016eef707", true, strategy);
            return;
        }

        //乐高mpos
        if(PROCESS_LEGAO_MPOS.equals(proccessName)){
            CrashReport.initCrashReport(getApplicationContext(), "a64996c710", true, strategy);
            return;
        }

        CrashReport.initCrashReport(getApplicationContext(), "82bf030e1b", true, strategy);
    }


    private ActivityLifecycleCallbacks mCallbacks = new ActivityLifecycleCallbacks() {

        @Override
        public void onActivityCreated(Activity activity, Bundle savedInstanceState) {
            LogFileUtils.writeLog(ContextUtils.getContext(), "\n" + DateUtils.format18(new Date()) + "########onActivityCreated() called with: activity = [" + activity + "]\n");
        }

        @Override
        public void onActivityStarted(Activity activity) {
            LogFileUtils.writeLog(ContextUtils.getContext(), "\n" + DateUtils.format18(new Date()) + "########onActivityStarted() called with: activity = [" + activity + "]\n");
        }

        @Override
        public void onActivityResumed(Activity activity) {
            LogFileUtils.writeLog(ContextUtils.getContext(), "\n" + DateUtils.format18(new Date()) + "########onActivityResumed() called with: activity = [" + activity + "]\n");
        }

        @Override
        public void onActivityPaused(Activity activity) {
            LogFileUtils.writeLog(ContextUtils.getContext(), "\n" + DateUtils.format18(new Date()) + "########onActivityPaused() called with: activity = [" + activity + "]\n");
        }

        @Override
        public void onActivityStopped(Activity activity) {
            LogFileUtils.writeLog(ContextUtils.getContext(), "\n" + DateUtils.format18(new Date()) + "########onActivityStopped() called with: activity = [" + activity + "]\n");
        }

        @Override
        public void onActivitySaveInstanceState(Activity activity, Bundle outState) {

        }

        @Override
        public void onActivityDestroyed(Activity activity) {
            LogFileUtils.writeLog(ContextUtils.getContext(), "\n" + DateUtils.format18(new Date()) + "########onActivityDestroyed() called with: activity = [" + activity + "]\n");
        }
    };


    /**
     * 动态设置Dansity
     *
     * @param context
     */
    public static void setCustomDensity(@NonNull Context context, @NonNull Activity activity) {
        final DisplayMetrics appDiaplayMetrics = activity.getResources().getDisplayMetrics();
        final float targetDensity = appDiaplayMetrics.widthPixels / 720;
        final int targetDensityDpi = (int) (160 * targetDensity);
        appDiaplayMetrics.density = appDiaplayMetrics.scaledDensity = targetDensity;
        appDiaplayMetrics.densityDpi = targetDensityDpi;
        final DisplayMetrics activityDisplayMetrics = context.getResources().getDisplayMetrics();
        activityDisplayMetrics.density = activityDisplayMetrics.scaledDensity = targetDensity;
        activityDisplayMetrics.densityDpi = targetDensityDpi;
    }






}
